/* Copyright(C) 2011 Cobac.Net All rights reserved. */

module PS2(
    input           CLK, RST,
    input   [1:0]   ADDR,
    input           WRITE,
    input   [7:0]   WRDATA,
    output  [7:0]   RDDATA,
    inout           PS2CLK, PS2DATA,     // o
    output reg      LOGCLK
);

/* VtgWX^AMWX^AtOށireg錾j */
reg     [9:0]   sft;
reg     [7:0]   ps2rdata;
reg     empty, valid;

/* Xe[g}Vip[^reg錾j */
parameter HALT=3'h0, CLKLOW=3'h1, STBIT=3'h2, SENDBIT=3'h3,
          WAITCLK=3'h4, GETBIT=3'h5, SETFLG=3'h6;
reg     [2:0]   cur, nxt;

/* M̃VtgWX^ݐM */
wire txregwr = (ADDR==2'h2) & WRITE;

/* WX^ǂݏo */
assign RDDATA =  (ADDR==2'h0) ? {6'b0, empty, valid}: ps2rdata;

/* PS2CLKo͂̃nU[hh~ */
reg     ps2clken;

always @( posedge CLK, posedge RST ) begin
    if ( RST )
        ps2clken <= 1'b0;
    else
        ps2clken <= (cur==CLKLOW  || cur==STBIT);
end

/* PS2o */
assign  PS2CLK  = (ps2clken) ? 1'b0  : 1'bz;
assign  PS2DATA = (cur==SENDBIT || cur==STBIT) ? sft[0]: 1'bz;

/* X^[grbgop100svpJE^ */
reg [12:0]   txcnt;

parameter TXMAX=13'd5000;
wire over100us = (txcnt==TXMAX-1);

always @( posedge CLK, posedge RST ) begin
    if ( RST )
        txcnt <= 13'h0000;
    else if ( cur==HALT )
        txcnt <= 13'h0000;
    else if ( over100us )
        txcnt <= 13'h0000;
    else
        txcnt <= txcnt + 13'h1;
end

/* MPS2CLK̗茟oѓ */
reg     [2:0]   sreg;
wire            clkfall;

always @( posedge CLK, posedge RST ) begin
    if ( RST )
        sreg <= 3'b000;
    else
        sreg <= {sreg[1:0], PS2CLK};
end

assign clkfall = sreg[2] & ~sreg[1];

/* MrbgJE^ */
reg     [3:0]   bitcnt;

always @( posedge CLK, posedge RST ) begin
    if ( RST )
        bitcnt <= 4'h0;
    else if ( cur==HALT )
        bitcnt <= 4'h0;
    else if ( (cur==SENDBIT || cur==GETBIT) & clkfall )
        bitcnt <= bitcnt + 4'h1;
end

/* Xe[g}V */
always @( posedge CLK, posedge RST ) begin
    if ( RST )
        cur <= HALT;
    else
        cur <= nxt;
end

always @* begin
    case ( cur )
        HALT:   if ( txregwr )
                    nxt <= CLKLOW;
                else if ( (PS2DATA==1'b0) & clkfall )
                    nxt <= GETBIT;
                else
                    nxt <= HALT;
        CLKLOW: if ( over100us )
                    nxt <= STBIT;
                else
                    nxt <= CLKLOW;
        STBIT:  if ( over100us )
                    nxt <= SENDBIT;
                else
                    nxt <= STBIT;
        SENDBIT:if ( (bitcnt==4'h9) & clkfall )
                    nxt <= WAITCLK;
                else
                    nxt <= SENDBIT;
        WAITCLK:if ( clkfall )
                    nxt <= HALT;
                else
                    nxt <= WAITCLK;
        GETBIT: if ( (bitcnt==4'h7) & clkfall )
                    nxt <= SETFLG;
                else
                    nxt <= GETBIT;
        SETFLG:  if ( clkfall )
                    nxt <= WAITCLK;
                else
                    nxt <= SETFLG;
        default:nxt <= HALT;
    endcase
end

/* emptytOiMemptyj */
always @( posedge CLK, posedge RST ) begin
    if ( RST )
        empty <= 1'b1;
    else
        empty <= (cur==HALT) ? 1'b1: 1'b0;
end

/* Mf[^LtO */
always @( posedge CLK, posedge RST ) begin
    if ( RST )
        valid <= 1'b0;
    else if ( (ADDR==2'h0) & WRITE )
        valid <= WRDATA[0];
    else if ( cur==SETFLG & clkfall )
        valid <= 1'b1;
end

/* VtgWX^ */
always @( posedge CLK, posedge RST ) begin
    if ( RST )
        sft <= 10'h000;
    else if ( txregwr )
        sft <= { ~(^WRDATA), WRDATA, 1'b0 };
    else if ( cur==SENDBIT & clkfall )
        sft <= {1'b1, sft[9:1]};
    else if ( cur==GETBIT & clkfall )
        sft <= {PS2DATA, sft[9:1]};
end

/* Mf[^ */
always @( posedge CLK, posedge RST ) begin
    if ( RST )
        ps2rdata <= 8'h00;
    else if ( cur==SETFLG & clkfall )
        ps2rdata <= sft[9:2];
end

/* SignalTap IIpNbNi1MHzj */
reg [4:0]   logcnt;

parameter MAX=5'd25;
wire cntend = (logcnt==MAX-1);

always @( posedge CLK, posedge RST ) begin
    if ( RST )
        logcnt <= 5'h00;
    else if ( cntend )
        logcnt <= 5'h00;
    else
        logcnt <= logcnt + 5'h1;
end

always @( posedge CLK, posedge RST ) begin
    if ( RST )
        LOGCLK <= 1'b0;
    else if ( cntend )
        LOGCLK <= ~LOGCLK;
end

endmodule
